#!/usr/bin/env python
"""
Entropy devices such as /dev/random or /dev/urandom read out data one
byte at a time.  Sometimes, however, entropy needed doesn't fall into
nice byte boundaries so valuable entropy is wasted.  On most
contemporary systems, cryptographic quality entropy is a scarce
resource.

This module provides two classes.  EntropyBuffer returns any number of
bits requested and saves extra entropy for the next call.  Tools
inherits from EntropyBuffer and provides useful functions.  Use Tools
to efficiently choose one of seventeen objects in a list while using
/dev/random efficiently.

Warning: While this code is fairly simple, it has also not been
statistically tested.  Use with caution.

"""

__version__ = "$Revision: 1.1 $"
__author__ = "Peter Hendrickson <pdh@wiredyne.com>"

# License:  This code is in the public domain.  Any of this code may be
#           freely reproduced or adapted, even without mentioning the
#           source.

# Warranty: IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR
#           DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
#           DAMAGES ARISING OUT OF THE USE OF THIS CODE, EVEN IF THE
#           AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#           THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES,
#           INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#           MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE
#           CODE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THERE
#           IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
#           SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

import struct
import types

class EntropyBuffer:

    """Buffer entropy so it is used efficiently."""

    def __init__(self, source):
        """Return EntropyBuffer object using source.

        If source is a string, it is treated as a random device such
        as /dev/random or /dev/urandom and opened.  If source is not a
        string, it is assumed to be an opened file object from which
        random bytes may be read.

        """
        if type(source) is types.StringType:
            self.device = source
            self.entropy_source = open(self.device, "r")
        else:
            self.device = None
            self.entropy_source = source

        self.bitpool = 0L
        self.bitcount = 0L
        self.bits_used = 0L
        self.bytes_read = 0L

    def __del__(self):
        """Close open entropy device, if necessary."""
        if self.device is not None:
            self.entropy_source.close()

    def entropy(self, bits):
        """Return a random integer in the interval [0,2^bits)."""
        if (type(bits) is not types.IntType) and \
           (type(bits) is not types.LongType):
            raise ValueError("Value of bits must be integer or long.")
        if type(bits) is types.IntType:
            bits = long(bits)
        if self.bitcount < bits:
            self.load_bitpool(bits)
        extrabits = self.bitcount - bits
        extrastuff = self.bitpool % 2L ** extrabits
        randval = (self.bitpool - extrastuff) >> extrabits
        self.bits_used += bits
        self.bitcount = extrabits
        self.bitpool = extrastuff
        return randval

    def load_bitpool(self, bits):
        """Load entropy so at least bits of entropy may be read."""
        if self.bitcount >= bits:
            return
        bits_needed = bits - self.bitcount
        full_bytes = int(bits_needed/8)
        extras = bits_needed % 8
        if( extras > 0 ):
            full_bytes += 1
        bytes_needed = full_bytes
        bytes_got = self.entropy_source.read(bytes_needed)
        if len(bytes_got) != bytes_needed:
            raise "Entropy source returned %d" % (bytes_got) + \
                  " bytes instead of expected %d bytes." %(bytes_needed)
        self.bytes_read += len(bytes_got)
        byte_list = struct.unpack("B" * bytes_needed, bytes_got)
        for byte in byte_list:
            self.bitpool *= 256
            self.bitpool += byte
            self.bitcount += 8

class Tools(EntropyBuffer):

    """Add useful functionality to EntropyBuffer.

    Provide a variety of functions which shuffle lists randomly,
    return random integers and so forth, all using the device.
    Commonly this will be /dev/random.

    """

    def randint(self, outcomes):
        """Return a random integer value in interval [0,outcomes)."""
        bits = log2(outcomes)
        randval = self.entropy(bits)
        while randval >= outcomes:
            randval = self.entropy(bits)
        return randval

    def random(self, resolution=40):
        """Return random number in interval [0.0,1.0).

        Optional argument resolution specifies how much entropy to use
        in generating the number.  Resolution defaults to 40 bits.

        """
        int_randval = self.entropy(resolution)
        float_randval = float(int_randval) / (2L ** resolution)
        return float_randval

    def choice(self, sequence):
        """Choose random element from sequence."""
        return sequence[self.randint(len(sequence))]

    def random_list(self, list, length):
        """Return length randomly selected elements from list."""
        return [self.choice(list) for i in range(length)]

    def shuffle(self, sequence):
        """Return copy of sequence in random order."""
        if len(sequence) == 1:
            return [sequence[0]]
        elif len(sequence) == 0:
            return []
        left = []
        right = []
        for item in sequence:
            randval = self.entropy(1)
            if randval:
                left.append(item)
            else:
                right.append(item)
        return self.shuffle(left) + self.shuffle(right)

def log2(number):
    """Return min bits needed to describe number."""
    number = abs(number)
    bits = 0
    while number:
        number >>= 1
        bits += 1
    return bits
