#!/usr/bin/env python
"""
Test the functionality of the EntropyBuffer module.  The only test
performed is an expected result test of the most basic functionality.
This module really needs statistical testing to do its job.

"""

__version__ = "$Revision: 1.1 $"
__author__ = "Peter Hendrickson <pdh@wiredyne.com>"

# License:  This code is in the public domain.  Any of this code may be
#           freely reproduced or adapted, even without mentioning the
#           source.

# Warranty: IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR
#           DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
#           DAMAGES ARISING OUT OF THE USE OF THIS CODE, EVEN IF THE
#           AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#           THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES,
#           INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#           MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE
#           CODE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THERE
#           IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
#           SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

import unittest
import StringIO
from EntropyBuffer import *

class EntropyBufferTest(unittest.TestCase):
    """Do basic test of EntropyBuffer class."""

    def testAllZeros(self):
        """Put all zeros in -- expect all zeros out."""
        srclen = 256
        for bitdraw in range(1,srclen):
            ntrpysrc = StringIO.StringIO('\x00' * srclen)
            ntrpy = EntropyBuffer(ntrpysrc)

            for count in xrange(1, int(srclen * 8 / bitdraw)):
                val = ntrpy.entropy(bitdraw)
                self.failUnless(val == 0)

    def testAllOnes(self):
        """Put all ones in -- expect all ones out."""
        srclen = 256
        for bitdraw in range(1,srclen):
            ntrpysrc = StringIO.StringIO('\xff' * srclen)
            ntrpy = EntropyBuffer(ntrpysrc)

            for count in xrange(1, int(srclen * 8 / bitdraw)):
                val = ntrpy.entropy(bitdraw)
                self.failUnless(val == (2L ** bitdraw - 1))

    def testPseudoRandomCase(self):
        """Tests EntropyBuffer with known pattern of bits.

        Takes a known pattern of bits which was originally randomly
        generated.  Then a series of bit reads of several sizes are
        made and compared with the values expected.  (The expected
        values were painstakingly generated by hand.)
        
        """

        # Known bit pattern
        # Hex:    5A94B9AC528B563C5AB3A7D23606ECF5DB4B036AE2FCAEEA
        #         5A6AF61ED01E34084C7DFC76BD8B0F2CE17C88FCE4ABD566
        #         7E10D6A8F03A8D410FB3C7495FEEAE89
        # Binary: 010110101001010010111001101011000101001010001011
        #         010101100011110001011010101100111010011111010010
        #         001101100000011011101100111101011101101101001011
        #         000000110110101011100010111111001010111011101010
        #         010110100110101011110110000111101101000000011110
        #         001101000000100001001100011111011111110001110110
        #         101111011000101100001111001011001110000101111100
        #         100010001111110011100100101010111101010101100110
        #         011111100001000011010110101010001111000000111010
        #         100011010100000100001111101100111100011101001001
        #         01011111111011101010111010001001
        pseudostring = 'Z\x94\xb9\xacR\x8bV<Z\xb3\xa7\xd26\x06\xec\xf5' + \
                       '\xdbK\x03j\xe2\xfc\xae\xeaZj\xf6\x1e\xd0\x1e4' + \
                       '\x08L}\xfcv\xbd\x8b\x0f,\xe1|\x88\xfc\xe4\xab' + \
                       '\xd5f~\x10\xd6\xa8\xf0:\x8dA\x0f\xb3\xc7I_\xee' + \
                       '\xae\x89'

        testcases = [[1, [0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0,
                          0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 0, 1, 1,
                          0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1,
                          0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1,
                          1, 1, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1,
                          1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1,
                          0, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0,
                          0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0,
                          1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1,
                          1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0,
                          1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0,
                          0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 1, 0,
                          1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0,
                          1, 1, 0, 1, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1,
                          1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1,
                          1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0,
                          0, 0, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0,
                          0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1,
                          0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0,
                          1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0,
                          1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1,
                          0, 1, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1,
                          1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1,
                          1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 0, 0,
                          1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0,
                          1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1,
                          1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0,
                          1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1,
                          0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0,
                          0, 1, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0,
                          0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1,
                          1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 1,
                          0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1,
                          0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0,
                          0, 1]],
                     [2, [1, 1, 2, 2, 2, 1, 1, 0, 2, 3, 2, 1, 2, 2, 3,
                          0, 1, 1, 0, 2, 2, 0, 2, 3, 1, 1, 1, 2, 0, 3,
                          3, 0, 1, 1, 2, 2, 2, 3, 0, 3, 2, 2, 1, 3, 3,
                          1, 0, 2, 0, 3, 1, 2, 0, 0, 1, 2, 3, 2, 3, 0,
                          3, 3, 1, 1, 3, 1, 2, 3, 1, 0, 2, 3, 0, 0, 0,
                          3, 1, 2, 2, 2, 3, 2, 0, 2, 3, 3, 3, 0, 2, 2,
                          3, 2, 3, 2, 2, 2, 1, 1, 2, 2, 1, 2, 2, 2, 3,
                          3, 1, 2, 0, 1, 3, 2, 3, 1, 0, 0, 0, 1, 3, 2,
                          0, 3, 1, 0, 0, 0, 2, 0, 1, 0, 3, 0, 1, 3, 3,
                          1, 3, 3, 3, 0, 1, 3, 1, 2, 2, 3, 3, 1, 2, 0,
                          2, 3, 0, 0, 3, 3, 0, 2, 3, 0, 3, 2, 0, 1, 1,
                          3, 3, 0, 2, 0, 2, 0, 3, 3, 3, 0, 3, 2, 1, 0,
                          2, 2, 2, 3, 3, 1, 1, 1, 1, 2, 1, 2, 1, 3, 3,
                          2, 0, 1, 0, 0, 3, 1, 1, 2, 2, 2, 2, 0, 3, 3,
                          0, 0, 0, 3, 2, 2, 2, 0, 3, 1, 1, 0, 0, 1, 0,
                          0, 3, 3, 2, 3, 0, 3, 3, 0, 1, 3, 1, 0, 2, 1,
                          1, 1, 3, 3, 3, 2, 3, 2, 2, 2, 3, 2, 2, 0, 2,
                          1]],
                     [3, [2, 6, 5, 1, 2, 2, 7, 1, 5, 3, 0, 5, 1, 2, 1,
                          3, 2, 5, 4, 3, 6, 1, 3, 2, 5, 4, 7, 2, 3, 7,
                          2, 2, 1, 5, 4, 0, 3, 3, 5, 4, 7, 5, 3, 5, 5,
                          5, 1, 3, 0, 0, 6, 6, 5, 3, 4, 2, 7, 7, 1, 2,
                          7, 3, 5, 2, 2, 6, 4, 6, 5, 3, 6, 6, 0, 7, 5,
                          5, 0, 0, 3, 6, 1, 5, 0, 0, 4, 1, 1, 4, 3, 7,
                          3, 7, 6, 1, 6, 6, 5, 7, 3, 0, 5, 4, 1, 7, 1,
                          3, 1, 6, 0, 5, 7, 4, 4, 2, 1, 7, 6, 3, 4, 4,
                          5, 2, 7, 5, 2, 5, 4, 6, 3, 7, 4, 1, 0, 3, 2,
                          6, 5, 2, 1, 7, 0, 0, 7, 2, 4, 3, 2, 4, 0, 4,
                          1, 7, 5, 4, 7, 4, 3, 5, 1, 1, 2, 7, 7, 6, 7,
                          2, 5, 6, 4, 2]],
                     [4, [5, 10, 9, 4, 11, 9, 10, 12, 5, 2, 8, 11, 5,
                          6, 3, 12, 5, 10, 11, 3, 10, 7, 13, 2, 3, 6,
                          0, 6, 14, 12, 15, 5, 13, 11, 4, 11, 0, 3, 6,
                          10, 14, 2, 15, 12, 10, 14, 14, 10, 5, 10, 6,
                          10, 15, 6, 1, 14, 13, 0, 1, 14, 3, 4, 0, 8,
                          4, 12, 7, 13, 15, 12, 7, 6, 11, 13, 8, 11,
                          0, 15, 2, 12, 14, 1, 7, 12, 8, 8, 15, 12,
                          14, 4, 10, 11, 13, 5, 6, 6, 7, 14, 1, 0, 13,
                          6, 10, 8, 15, 0, 3, 10, 8, 13, 4, 1, 0, 15,
                          11, 3, 12, 7, 4, 9, 5, 15, 14, 14, 10, 14,
                          8, 9]],
                     [5, [11, 10, 10, 11, 19, 11, 2, 18, 17, 13, 11,
                          3, 24, 22, 21, 19, 20, 31, 9, 3, 12, 1, 23,
                          12, 30, 23, 13, 20, 22, 0, 27, 10, 28, 11,
                          30, 10, 29, 26, 18, 26, 13, 11, 27, 1, 29,
                          20, 0, 30, 6, 16, 4, 4, 24, 31, 15, 28, 14,
                          26, 30, 24, 22, 3, 25, 12, 28, 5, 30, 8, 17,
                          31, 7, 4, 21, 15, 10, 22, 12, 31, 16, 16,
                          26, 26, 20, 15, 0, 14, 20, 13, 8, 4, 7, 27,
                          7, 17, 26, 9, 11, 31, 23, 10, 29, 2]],
                     [6, [22, 41, 18, 57, 43, 5, 10, 11, 21, 35, 49, 26, 44,
                          58, 31, 18, 13, 32, 27, 44, 61, 29, 45, 11, 0, 54,
                          43, 34, 63, 10, 59, 42, 22, 38, 43, 54, 7, 45, 0, 30,
                          13, 0, 33, 12, 31, 31, 49, 54, 47, 24, 44, 15, 11,
                          14, 5, 60, 34, 15, 51, 36, 42, 61, 21, 38, 31, 33, 3,
                          22, 42, 15, 0, 58, 35, 20, 4, 15, 44, 60, 29, 9, 23,
                          62, 58, 46, 34]],
                     [7, [45, 37, 23, 26, 98, 74, 22, 86, 30, 22, 86, 58, 62,
                          72, 108, 6, 118, 61, 59, 52, 88, 13, 85, 98, 126, 43,
                          93, 37, 83, 43, 108, 30, 104, 7, 70, 64, 66, 49, 123,
                          124, 59, 47, 49, 48, 121, 51, 66, 124, 68, 63, 28,
                          74, 94, 85, 76, 126, 8, 53, 85, 15, 1, 106, 26, 65,
                          7, 108, 120, 116, 74, 127, 93, 46, 68]],
                     [8, [90, 148, 185, 172, 82, 139, 86, 60, 90, 179, 167,
                          210, 54, 6, 236, 245, 219, 75, 3, 106, 226, 252, 174,
                          234, 90, 106, 246, 30, 208, 30, 52, 8, 76, 125, 252,
                          118, 189, 139, 15, 44, 225, 124, 136, 252, 228, 171,
                          213, 102, 126, 16, 214, 168, 240, 58, 141, 65, 15,
                          179, 199, 73, 95, 238, 174, 137]],
                     [9, [181, 82, 461, 197, 81, 213, 286, 90, 359, 159, 145,
                          352, 221, 317, 237, 331, 6, 427, 279, 458, 477, 150,
                          309, 246, 61, 320, 241, 320, 265, 287, 254, 118, 379,
                          44, 121, 206, 47, 290, 126, 228, 343, 341, 307, 481,
                          26, 426, 120, 58, 282, 260, 125, 316, 233, 87, 503,
                          174]],
                     [10, [362, 331, 619, 82, 557, 355, 790, 691, 671, 291,
                           385, 748, 983, 436, 704, 874, 907, 970, 954, 602,
                           427, 865, 948, 30, 208, 132, 799, 508, 474, 984,
                           707, 812, 901, 968, 575, 228, 687, 342, 415, 528,
                           858, 655, 14, 653, 260, 251, 241, 841, 383, 746,
                           930]],
                     [11, [724, 1326, 856, 1320, 1451, 241, 854, 935, 1681,
                           1409, 1497, 1885, 1445, 1037, 1372, 764, 1399, 662,
                           1237, 1889, 1896, 120, 1665, 76, 1007, 1821, 1403,
                           176, 1942, 901, 1937, 252, 1829, 757, 716, 2017,
                           107, 675, 1543, 653, 520, 1004, 1934, 1173, 2039,
                           698]],
                     [12, [1449, 1209, 2757, 651, 1379, 3162, 2874, 2002, 864,
                           1772, 3933, 2891, 54, 2786, 4042, 3818, 1446, 2806,
                           493, 30, 832, 2124, 2015, 3190, 3032, 2831, 718,
                           380, 2191, 3300, 2749, 1382, 2017, 214, 2703, 58,
                           2260, 271, 2876, 1865, 1534, 3758]],
                     [13, [2898, 4838, 5673, 2229, 3192, 5804, 7486, 4662, 221,
                           5079, 3493, 4150, 5573, 7979, 5970, 6762, 7875,
                           6976, 3866, 132, 6395, 7965, 5612, 2831, 1436, 1522,
                           1150, 3658, 6058, 6559, 4230, 5800, 7687, 2613, 135,
                           6972, 3730, 6139, 5492]],
                     [14, [5797, 2970, 12618, 2902, 3862, 11066, 8008, 13830,
                           15165, 7604, 11277, 10978, 16171, 11941, 10667,
                           13854, 13319, 9024, 8497, 15868, 7599, 6320, 15539,
                           8572, 8767, 3658, 12117, 9854, 1077, 10895, 234,
                           3393, 1004, 15476, 9599, 11950]],
                     [15, [11594, 11883, 2641, 13667, 25301, 20127, 9324, 1772,
                           31469, 21184, 27996, 12234, 30546, 27051, 27709,
                           20510, 6660, 4895, 16270, 27608, 22649, 13189,
                           30993, 31972, 21994, 22943, 16922, 27279, 468,
                           13572, 8039, 18249, 12279, 11170]],
                     [16, [23188, 47532, 21131, 22076, 23219, 42962, 13830,
                           60661, 56139, 874, 58108, 44778, 23146, 63006,
                           53278, 13320, 19581, 64630, 48523, 3884, 57724,
                           35068, 58539, 54630, 32272, 54952, 61498, 36161,
                           4019, 51017, 24558, 44681]],
                     [17, [46377, 59057, 37978, 91077, 87668, 128141, 66422,
                           62939, 38406, 109451, 124279, 42406, 89795, 111623,
                           72196, 19581, 129261, 63020, 31079, 6088, 73628,
                           76533, 45887, 4310, 86496, 59957, 2173, 81012,
                           76797]]
                     ]

        for case in testcases:
            ntrpysrc = StringIO.StringIO(pseudostring)
            ntrpy = EntropyBuffer(ntrpysrc)
            bitdraw = case[0]
            results = case[1]
            count = 0
            for result in results:
                val = ntrpy.entropy(bitdraw)
                self.failUnless( val == result )
                count += 1

                min_bytes_read = int(bitdraw * count / 8)
                extrabits = bitdraw * count % 8
                if extrabits > 0:
                    bytes_read = min_bytes_read + 1
                else:
                    bytes_read = min_bytes_read

                self.failUnless(ntrpy.bits_used == bitdraw * count)
                self.failUnless(ntrpy.bytes_read == bytes_read)

if __name__ == '__main__':
    unittest.main()
